'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _string = require('string');

var _string2 = _interopRequireDefault(_string);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var debug = (0, _debug2.default)('Concept');

var pushUnique = function pushUnique(array, item) {
  if (array.indexOf(item) === -1) {
    array.push(item);
    return true;
  }
  return false;
};

var ConceptObj = function () {
  function ConceptObj(db) {
    var name = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'no-name';

    _classCallCheck(this, ConceptObj);

    this.name = name;
    this.highLevelConcepts = [];
    this.referencedConcepts = [];
    this.tables = [];
    this.db = db;
  }

  _createClass(ConceptObj, [{
    key: 'createFact',
    value: function createFact(subject, verb, object, callback) {
      debug('Creating new fact', subject, verb, object);
      this.db.put({ subject: subject, predicate: verb, object: object }, function (err) {
        callback(err);
      });
    }
  }]);

  return ConceptObj;
}();

var readFile = function readFile(file, db, cb) {
  var tableArgs = [];
  var factDef = [];
  var inTableData = false;

  var c = new ConceptObj(db, file);

  var isConcept = function isConcept(term) {
    if (_lodash2.default.isString(term) || _lodash2.default.isString(term.s)) {
      return term.indexOf('~') !== -1;
    }
    return false;
  };

  var prepConcept = function prepConcept(term) {
    return (0, _string2.default)(term).chompLeft('~').toLowerCase().trim().s;
  };

  var findOrCreateConcept = function findOrCreateConcept(term) {
    if (isConcept(term)) {
      pushUnique(c.referencedConcepts, prepConcept(term));
    }

    var nc = term.chompLeft('~').toLowerCase().trim();
    return nc.s;
  };

  var parseArray = function parseArray(line) {
    var buff = [];
    var inQuotes = false;
    var ar = [];
    for (var n = 0; n < line.length; n++) {
      if (/[~a-z0-9-_']/i.test(line[n]) || inQuotes) {
        buff.push(line[n]);
        if (line[n] === '"') {
          buff.pop();
          inQuotes = false;
        }
      } else if (line[n] === '"') {
        inQuotes = true;
      } else {
        if (buff.join('') !== '') {
          ar.push(buff.join(''));
        }
        buff = [];
      }
    }
    // Flush the remaining items from the buffer
    if (buff.length !== 0) {
      ar.push(buff.join(''));
    }

    return ar;
  };

  var factCount = 0;

  var addFact = function addFact(def, callback) {
    if (factDef) {
      _async2.default.timesSeries(factDef.length, function (i, next) {
        if (factDef[i].length === 3) {
          (function () {
            var p0 = def[factDef[i][0]] ? def[factDef[i][0]] : factDef[i][0];
            var p1 = def[factDef[i][1]] ? def[factDef[i][1]] : factDef[i][1];
            var p2 = def[factDef[i][2]] ? def[factDef[i][2]] : factDef[i][2];

            if (_lodash2.default.isArray(p0) || _lodash2.default.isArray(p2)) {
              if (_lodash2.default.isArray(p0)) {
                _async2.default.timesSeries(p0.length, function (n, next2) {
                  if (isConcept(p0[n])) pushUnique(c.referencedConcepts, prepConcept(p0[n]));
                  if (isConcept(p1)) pushUnique(c.referencedConcepts, prepConcept(p1));
                  if (isConcept(p2)) pushUnique(c.referencedConcepts, prepConcept(p2));
                  c.createFact(p0[n], p1, prepConcept(p2), function (err) {
                    factCount += 1;
                    next2(err);
                  });
                }, function (err) {
                  next(err);
                });
              }
              if (_lodash2.default.isArray(p2)) {
                _async2.default.timesSeries(p2.length, function (n, next2) {
                  if (isConcept(p0)) pushUnique(c.referencedConcepts, prepConcept(p0));
                  if (isConcept(p1)) pushUnique(c.referencedConcepts, prepConcept(p1));
                  if (isConcept(p2[n])) pushUnique(c.referencedConcepts, prepConcept(p2[n]));
                  c.createFact(p0, p1, prepConcept(p2[n]), function (err) {
                    factCount += 1;
                    next2(err);
                  });
                }, function (err) {
                  next(err);
                });
              }
            } else {
              if (isConcept(p0)) pushUnique(c.referencedConcepts, prepConcept(p0));
              if (isConcept(p1)) pushUnique(c.referencedConcepts, prepConcept(p1));
              if (isConcept(p2)) pushUnique(c.referencedConcepts, prepConcept(p2));

              c.createFact(p0, p1, prepConcept(p2), function (err) {
                factCount += 1;
                next(err);
              });
            }
          })();
        }
      }, function (err) {
        return callback(err);
      });
    }
  };

  var addTableData = function addTableData(line, callback) {
    var buff = [];
    var conceptsInBraces = false;
    var inQuotes = false;
    var inBraces = false;
    var i = 0;
    var data = {};
    var nconcept = void 0;
    if (line === '') {
      return callback(null);
    }
    _async2.default.timesSeries(line.length, function (n, next) {
      if (/[~a-z0-9-._']/i.test(line[n]) || inQuotes || inBraces) {
        buff.push(line[n]);
        if (line[n] === '"') {
          inQuotes = false;
          buff.pop();
        } else if (line[n] === ']') {
          inBraces = false;
          buff.pop();
        }
      } else if (line[n] === '"') {
        inQuotes = true;
      } else if (line[n] === '[') {
        inBraces = true;
        conceptsInBraces = true;
      } else {
        // If there's a space and we're not within quotes or braces
        nconcept = buff.join('');

        if (nconcept !== '') {
          if (conceptsInBraces) {
            data[tableArgs[i]] = parseArray(nconcept);
          } else {
            data[tableArgs[i]] = nconcept;
          }
          conceptsInBraces = false;
          i += 1;
          buff = [];
          if (i === tableArgs.length) {
            return addFact(data, function (err) {
              next(err);
            });
          }
        }
      }
      next(null);
    }, function (err) {
      if (buff.length !== 0) {
        nconcept = buff.join('');
        if (conceptsInBraces) {
          data[tableArgs[i]] = parseArray(nconcept);
        } else {
          data[tableArgs[i]] = nconcept;
        }
        i += 1;

        if (i === tableArgs.length) {
          return addFact(data, function (err) {
            callback(err);
          });
        }
        return callback(err);
      }
      return callback(err);
    });
  };

  // Adds the Concept line, minus the main concept
  var addConcepts = function addConcepts(conceptName, line, callback) {
    var buff = [];
    var inQuotes = false;
    var l2 = (0, _string2.default)(line).between('(');
    var comment = (0, _string2.default)(l2.s).indexOf('#');

    var eol = l2.length;
    if (comment !== -1) {
      eol = comment - 1;
    }

    _async2.default.timesSeries(eol, function (n, next) {
      if (/[~a-z0-9-_']/i.test(l2.s[n]) || inQuotes) {
        buff.push(l2.s[n]);
        if (l2.s[n] === '"') {
          buff.pop();
          inQuotes = false;
        }
      } else if (l2.s[n] === '"') {
        inQuotes = true;
      } else {
        var nconcept = buff.join('');
        buff = [];
        if (nconcept !== '' && conceptName) {
          var _ret2 = function () {
            var cleanConcept = prepConcept(conceptName);
            var cleanRel = prepConcept(nconcept);
            return {
              v: c.createFact(cleanConcept, 'example', cleanRel, function (err) {
                c.createFact(cleanRel, 'isa', cleanConcept, function (err) {
                  if (err) {
                    console.log(err);
                  }
                  next(err);
                });
              })
            };
          }();

          if ((typeof _ret2 === 'undefined' ? 'undefined' : _typeof(_ret2)) === "object") return _ret2.v;
        }
      }
      return next(null);
    }, function (err) {
      if (err) {
        console.log(err);
      }
      callback(err);
    });
  };

  var input = _fs2.default.createReadStream(file);
  var remaining = '';
  var currentConcept = void 0;

  // This exists so that the 'end' handler doesn't fire until all the callbacks
  // from the 'data' handler have finished.
  var callbackComplete = false;

  input.on('data', function (data) {
    remaining += data;

    // This is async since it can potentially call addTableData or addConcepts
    _async2.default.whilst(function () {
      return remaining.indexOf('\n') > -1;
    }, function (callback) {
      var index = remaining.indexOf('\n');

      var line = remaining.substring(0, index);
      remaining = remaining.substring(index + 1);

      var nline = (0, _string2.default)(line).trimLeft();
      if (nline.indexOf('concept:') === 0) {
        // If the line starts with 'concept:', start off a new concept
        var conceptName = (0, _string2.default)(nline).between('concept:', '(').trim();
        // Some lines have extra leading descriptors
        conceptName = conceptName.split(' ')[0];

        if (isConcept(conceptName)) {
          pushUnique(c.highLevelConcepts, prepConcept(conceptName));
        }

        currentConcept = findOrCreateConcept((0, _string2.default)(conceptName));
        return addConcepts(currentConcept, nline, callback);
      } else if (currentConcept) {
        // Otherwise continue with the currently importing concept (more to import)
        return addConcepts(currentConcept, nline, callback);
      } else if (nline.indexOf('table:') === 0) {
        // Reset the Fact Tables
        factDef = [];
        inTableData = false;
        var _conceptName = (0, _string2.default)(nline).between('table:', '(').trim();
        _conceptName = _conceptName.split(' ')[0];
        debug('Table', _conceptName);
        pushUnique(c.tables, prepConcept(_conceptName));
        tableArgs = (0, _string2.default)(nline).between('(', ')').trim().split(' ').filter(function (e) {
          return e;
        });
        debug('TableArgs', tableArgs);
      } else if (inTableData) {
        // Skip comments
        if (nline.indexOf('#') === -1) {
          return addTableData(nline.s, callback);
        }
      } else if (nline.indexOf('^createfact') === 0) {
        factDef.push((0, _string2.default)(nline).between('(', ')').trim().split(' '));
        debug('factDef', factDef);
      } else if (nline.indexOf('DATA:') === 0) {
        inTableData = true;
      }
      return callback(null);
    }, function (err) {
      if (err) {
        console.log(err);
      }
      callbackComplete = true;
    });
  });

  input.on('end', function () {
    // Wait for callbacks in 'data' handler to finish.
    _async2.default.whilst(function () {
      return !callbackComplete;
    }, function (callback) {
      setTimeout(function () {
        return callback();
      }, 100);
    }, function () {
      if (remaining.length > 0) {
        var nline = (0, _string2.default)(remaining).trimLeft();

        if (nline.indexOf('concept:') === 0) {
          var conceptName = (0, _string2.default)(nline).between('concept:', '(').trim();
          // Some lines have extra leading descriptors
          conceptName = conceptName.split(' ')[0];

          if (isConcept(conceptName)) {
            pushUnique(c.highLevelConcepts, prepConcept(conceptName));
          }

          currentConcept = findOrCreateConcept((0, _string2.default)(conceptName));
          return addConcepts(currentConcept, nline, function () {
            debug('Added # Facts', factCount);
            cb(c);
          });
        }

        return addConcepts(currentConcept, nline, function () {
          debug('Added # Facts', factCount);
          cb(c);
        });
      }
      debug('Added # Facts', factCount);
      return cb(c);
    });
  });
};

// This is the same as read file, but it reads an array of files
var readFiles = function readFiles(files, db, callback) {
  var itor = function itor(file, next) {
    readFile(file, db, function (conceptObj) {
      // Add the reference concepts
      _async2.default.times(conceptObj.referencedConcepts.length, function (n, next2) {
        conceptObj.createFact(conceptObj.referencedConcepts[n], 'isa', 'concept', function (err) {
          next2(err);
        });
      }, function (err) {
        if (err) {
          console.log(err);
        }
        next(null, conceptObj);
      });
    });
  };

  _async2.default.mapSeries(files, itor, function (err, conceptMap) {
    callback(err, conceptMap);
  });
};

exports.default = {
  readFile: readFile,
  readFiles: readFiles
};