'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

var _levelgraph = require('levelgraph');

var _levelgraph2 = _interopRequireDefault(_levelgraph);

var _levelSublevel = require('level-sublevel');

var _levelSublevel2 = _interopRequireDefault(_levelSublevel);

var _concepts = require('./concepts');

var _concepts2 = _interopRequireDefault(_concepts);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// We wrap the DB with extra functions.
var expand = function expand(db, level) {
  var createUserDB = function createUserDB(dbName) {
    // With Hooks
    var sub = (0, _levelSublevel2.default)(level);
    var graphLevel = sub.sublevel(dbName);
    graphLevel.db = level.db;
    graphLevel.approximateSize = level.db.approximateSize.bind(level.db);

    var graph = (0, _levelgraph2.default)(graphLevel);

    return expand(graph);
  };

  var createUserDBWithData = function createUserDBWithData(dbName, files, cb) {
    var sub = (0, _levelSublevel2.default)(level);
    var graphLevel = sub.sublevel(dbName);
    graphLevel.db = level.db;
    graphLevel.approximateSize = level.db.approximateSize.bind(level.db);

    var graph = (0, _levelgraph2.default)(graphLevel);

    _concepts2.default.readFiles(files, graph, function (err, conceptMap) {
      cb(err, expand(graph));
    });
  };

  // This adds a file to an existing DB
  var loadFiles = function loadFiles(files, cb) {
    _concepts2.default.readFiles(files, db, cb);
  };

  var conceptToList = function conceptToList(term, depth, cb) {
    if (_lodash2.default.isFunction(depth)) {
      cb = depth;
      depth = 0;
    }

    if (depth === 10) {
      return cb('DEPTH', null);
    }

    db.search([{ subject: term, predicate: 'example', object: db.v('concepts') }], function (err, results) {
      if (err) {
        console.log(err);
      }
      var itor = function itor(itemx, next) {
        // db.get({subject: itemx , predicate: 'isa', object: 'concept' },
        db.search([{ subject: itemx, predicate: 'isa', object: db.v('concepts') }, { subject: db.v('concepts'), predicate: 'isa', object: 'concept' }, { subject: itemx, predicate: 'example', object: db.v('term') }], function (err, res2) {
          if (err) {
            console.log(err);
          }
          if (_lodash2.default.isEmpty(res2)) {
            next(null, itemx);
          } else if (itemx !== term) {
            conceptToList(itemx, depth + 1, next);
          } else {
            next(null, itemx);
          }
        });
      };

      var list = results.map(function (item) {
        return item.concepts;
      });

      _async2.default.map(list, itor, function (err, res) {
        if (err) {
          console.log(err);
        }
        var resultSet = _lodash2.default.uniq(_lodash2.default.flatten(res));
        cb(null, resultSet);
      });
    });
  };

  var findParentConcepts = function findParentConcepts(term, cb) {
    db.search([{ subject: term, predicate: 'isa', object: db.v('concepts') }], function (err, results) {
      if (err) {
        console.log(err);
      }
      if (results.length !== 0) {
        var cl = results.map(function (item) {
          return item.concepts;
        });
        cb(null, cl);
      } else {
        cb(null, []);
      }
    });
  };

  return {
    conceptToList: conceptToList,
    createUserDB: createUserDB,
    createUserDBWithData: createUserDBWithData,
    db: db,
    findParentConcepts: findParentConcepts,
    level: level,
    loadFiles: loadFiles
  };
};

exports.default = expand;