'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _levelup = require('levelup');

var _levelup2 = _interopRequireDefault(_levelup);

var _mongodown = require('mongodown');

var _mongodown2 = _interopRequireDefault(_mongodown);

var _mongojs = require('mongojs');

var _mongojs2 = _interopRequireDefault(_mongojs);

var _levelgraph = require('levelgraph');

var _levelgraph2 = _interopRequireDefault(_levelgraph);

var _levelgraphRecursive = require('levelgraph-recursive');

var _levelgraphRecursive2 = _interopRequireDefault(_levelgraphRecursive);

var _expand = require('./expand');

var _expand2 = _interopRequireDefault(_expand);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var clean = function clean(dbName, cb) {
  var db = (0, _mongojs2.default)(dbName);
  db.dropDatabase(function (err) {
    db.close();
    cb(err);
  });
};

var create = function create(dbName, cleanDb, cb) {
  var postClean = function postClean() {
    return (0, _levelup2.default)(dbName, { db: _mongodown2.default }, function (err, leveldb) {
      if (err) {
        return cb(err);
      }
      var db = (0, _levelgraphRecursive2.default)((0, _levelgraph2.default)(leveldb));
      return cb(err, (0, _expand2.default)(db, leveldb));
    });
  };

  if (cleanDb) {
    return clean(dbName, function (err) {
      if (err) {
        return cb(err);
      }
      return postClean();
    });
  }
  return postClean();
};

// Same as create except you can load additional data into the system
var load = function load(dbName, files, cleanDb, cb) {
  create(dbName, cleanDb, function (err, db) {
    if (err) {
      return cb(err);
    }
    return db.loadFiles(files, function (err2) {
      cb(err2, db);
    });
  });
};

exports.default = {
  clean: clean,
  create: create,
  load: load
};